import { CommandInteraction, EmbedBuilder, SlashCommandBuilder } from "discord.js";
import { Command } from "../type/command";
import { CoreClient } from "../client/client";
import EmbedColours from "../constants/EmbedColours";
import AppLogger from "../client/appLogger";

export default class Series extends Command {
    constructor() {
        super();

        this.CommandBuilder = new SlashCommandBuilder()
            .setName("view")
            .setDescription("View details on a series")
            .addSubcommand(x =>
                x
                    .setName("view")
                    .setDescription("View a specifiic series by id")
                    .addStringOption(y =>
                        y
                            .setName("id")
                            .setDescription("The series id")
                            .setRequired(true)))
            .addSubcommand(x =>
                x
                    .setName("list")
                    .setDescription("List all series"));
    }

    public override async execute(interaction: CommandInteraction) {
        if (!interaction.isChatInputCommand()) return;

        switch (interaction.options.getSubcommand()) {
            case "view":
                await this.ViewSeries(interaction);
                break;
            case "list":
                await this.ListSeries(interaction);
                break;
            default:
                AppLogger.LogWarn("Commands/Series", `Subcommand doesn't exist: ${interaction.options.getSubcommand()}`);
                await interaction.reply("Subcommand doesn't exist.");
        }
    }

    private async ViewSeries(interaction: CommandInteraction) {
        const id = interaction.options.get("id");

        AppLogger.LogSilly("Commands/Series/View", `Parameters: id=${id?.value}`);

        if (!id) return;

        const series = CoreClient.Cards.find(x => x.id == id.value);

        if (!series) {
            AppLogger.LogVerbose("Commands/Series/View", "Series not found.");

            await interaction.reply("Series not found.");
            return;
        }

        const description = series.cards
            .map(x => { return `[${x.id}] ${x.name}` })
            .join("\n");

        const embed = new EmbedBuilder()
            .setTitle(series.name)
            .setColor(EmbedColours.Ok)
            .setDescription(description)
            .setFooter({ text: `${series.id} · ${series.cards.length} cards` });

        await interaction.reply({ embeds: [ embed ]});
    }

    private async ListSeries(interaction: CommandInteraction) {
        const description = CoreClient.Cards
            .map(x => { return `[${x.id}] ${x.name}` })
            .join("\n");

        const embed = new EmbedBuilder()
            .setTitle("Series")
            .setColor(EmbedColours.Ok)
            .setDescription(description)
            .setFooter({ text: `${CoreClient.Cards.length} series` });

        await interaction.reply({ embeds: [ embed ]});
    }
}